@@ -79,6 +79,7 @@ use std::{
 
 use async_trait::async_trait;
 use deadpool_runtime::Runtime;
+use retain_mut::RetainMut;
 use tokio::sync::{Semaphore, TryAcquireError};
 
 pub use crate::Status;
@@ -301,7 +302,7 @@ impl<M: Manager, W: From<Object<M>>> Pool<M, W> {
     ///
     /// See [`PoolError`] for details.
     pub async fn get(&self) -> Result<W, PoolError<M::Error>> {
-        self.timeout_get(&self.inner.config.timeouts).await
+        self.timeout_get(&self.timeouts()).await
     }
 
     /// Retrieves an [`Object`] from this [`Pool`] and doesn't wait if there is
@@ -311,10 +312,16 @@ impl<M: Manager, W: From<Object<M>>> Pool<M, W> {
     /// # Errors
     ///
     /// See [`PoolError`] for details.
+    #[deprecated(
+        since = "0.9.3",
+        note = "The name of this method is highly misleading. Please use timeout_get instead. e.g. `pool.timeout_get(&Timeouts { wait: Some(Duration::ZERO), ..pool.timeouts() })"
+    )]
     pub async fn try_get(&self) -> Result<W, PoolError<M::Error>> {
-        let mut timeouts = self.inner.config.timeouts;
-        timeouts.wait = Some(Duration::from_secs(0));
-        self.timeout_get(&timeouts).await
+        self.timeout_get(&Timeouts {
+            wait: Some(Duration::ZERO),
+            ..self.timeouts()
+        })
+        .await
     }
 
     /// Retrieves an [`Object`] from this [`Pool`] using a different `timeout`
@@ -343,7 +350,7 @@ impl<M: Manager, W: From<Object<M>>> Pool<M, W> {
             apply_timeout(
                 self.inner.runtime,
                 TimeoutType::Wait,
-                self.inner.config.timeouts.wait,
+                timeouts.wait,
                 async {
                     self.inner
                         .semaphore
@@ -369,12 +376,12 @@ impl<M: Manager, W: From<Object<M>>> Pool<M, W> {
                     let _ = self.inner.available.fetch_sub(1, Ordering::Relaxed);
                 });
 
-                // Apply post_recycle hooks
+                // Apply pre_recycle hooks
                 if let Some(_e) = self
                     .inner
                     .hooks
                     .pre_recycle
-                    .apply(&mut obj.inner.as_mut().unwrap(), PoolError::PreRecycleHook)
+                    .apply(obj.inner.as_mut().unwrap(), PoolError::PreRecycleHook)
                     .await?
                 {
                     continue;
@@ -383,7 +390,7 @@ impl<M: Manager, W: From<Object<M>>> Pool<M, W> {
                 if apply_timeout(
                     self.inner.runtime,
                     TimeoutType::Recycle,
-                    self.inner.config.timeouts.recycle,
+                    timeouts.recycle,
                     self.inner.manager.recycle(&mut *obj),
                 )
                 .await
@@ -397,7 +404,7 @@ impl<M: Manager, W: From<Object<M>>> Pool<M, W> {
                     .inner
                     .hooks
                     .post_recycle
-                    .apply(&mut obj.inner.as_mut().unwrap(), PoolError::PostRecycleHook)
+                    .apply(obj.inner.as_mut().unwrap(), PoolError::PostRecycleHook)
                     .await?
                 {
                     continue;
@@ -416,7 +423,7 @@ impl<M: Manager, W: From<Object<M>>> Pool<M, W> {
                         obj: apply_timeout(
                             self.inner.runtime,
                             TimeoutType::Create,
-                            self.inner.config.timeouts.create,
+                            timeouts.create,
                             self.inner.manager.create(),
                         )
                         .await?,
@@ -431,7 +438,7 @@ impl<M: Manager, W: From<Object<M>>> Pool<M, W> {
                     .inner
                     .hooks
                     .post_create
-                    .apply(&mut obj.inner.as_mut().unwrap(), PoolError::PostCreateHook)
+                    .apply(obj.inner.as_mut().unwrap(), PoolError::PostCreateHook)
                     .await?
                 {
                     continue;
@@ -492,6 +499,48 @@ impl<M: Manager, W: From<Object<M>>> Pool<M, W> {
         }
     }
 
+    /// Retains only the objects specified by the given function.
+    ///
+    /// This function is typically used to remove objects from
+    /// the pool based on their current state or metrics.
+    ///
+    /// **Caution:** This function blocks the entire pool while
+    /// it is running. Therefore the given function should not
+    /// block.
+    ///
+    /// The following example starts a background task that
+    /// runs every 30 seconds and removes objects from the pool
+    /// that haven't been used for more than one minute.
+    ///
+    /// ```rust,ignore
+    /// let interval = Duration::from_secs(30);
+    /// let max_age = Duration::from_secs(60);
+    /// tokio::spawn(async move {
+    ///     loop {
+    ///         tokio::time::sleep(interval).await;
+    ///         pool.retain(|_, metrics| metrics.last_used() < max_age);
+    ///     }
+    /// });
+    /// ```
+    pub fn retain(&self, f: impl Fn(&M::Type, Metrics) -> bool) {
+        let mut guard = self.inner.slots.lock().unwrap();
+        let len_before = guard.vec.len();
+        RetainMut::retain_mut(&mut guard.vec, |obj| {
+            if f(&obj.obj, obj.metrics) {
+                true
+            } else {
+                self.manager().detach(&mut obj.obj);
+                false
+            }
+        });
+        guard.size -= len_before - guard.vec.len();
+    }
+
+    /// Get current timeout configuration
+    pub fn timeouts(&self) -> Timeouts {
+        self.inner.config.timeouts
+    }
+
     /// Closes this [`Pool`].
     ///
     /// All current and future tasks waiting for [`Object`]s will return
